@@ -226,10 +226,9 @@ extern int md5Reset(register md5Param* p);
 extern int md5Update(md5Param* p, const byte* data, int size);
 extern int md5Digest(md5Param* p, uint32* data);
 
-static char db_name[64]     = "samhain"; 
-static char db_table[64]    = "log"; 
-
-static char db_host[64]     = "localhost"; 
+static char db_name[64]     = ""; 
+static char db_table[64]    = ""; 
+static char db_host[64]     = ""; 
 static char db_user[64]     = ""; 
 static char db_password[64] = "";
 
@@ -278,7 +277,13 @@ int sh_database_set_password (char * str)
  *  Postgresql stuff, completely untested, coded by the docs
  *
  ******************************************************************/
+
+#ifdef HAVE_PGSQL_LIBPQ_FE_H
+#include <pgsql/libpq-fe.h>
+#else
 #include <libpq-fe.h>
+#endif
+
 int sh_database_query (char  * query, long * id)
 {
   char     * p;
@@ -291,6 +296,11 @@ int sh_database_query (char  * query, long * id)
 
   p = &conninfo[0]; 
 
+  if (db_host[0]     == '\0')
+    sl_strlcpy(db_host,  _("localhost"), 64);
+  if (db_name[0]     == '\0')
+    sl_strlcpy(db_name,  _("samhain"),   64);
+
   if (db_host[0]     != '\0' && NULL != strchr(db_host, '.')) 
     {
       sl_snprintf(p, 255, "hostaddr=%s ", db_host);
@@ -356,7 +366,13 @@ int sh_database_query (char  * query, long * id)
 
 
 #ifdef WITH_MYSQL 
+
+#ifdef HAVE_MYSQL_MYSQL_H
+#include <mysql/mysql.h>
+#else
 #include <mysql.h>
+#endif 
+
 int sh_database_query (char  * query, long * id)
 {
   char *     p;
@@ -365,6 +381,11 @@ int sh_database_query (char  * query, long * id)
 
   SL_ENTER(_("sh_database_query"));
 
+  if (db_host[0]     == '\0')
+    sl_strlcpy(db_host,  _("localhost"), 64);
+  if (db_name[0]     == '\0')
+    sl_strlcpy(db_name,  _("samhain"),   64);
+
   if (NULL == mysql_init(&db_conn))
     {
       p = NULL; status = 0;
@@ -486,17 +507,6 @@ long sh_database_entry (dbins * db_entry, long id)
     sprintf(num, "%ld", id);
 
 
-  /*
-  sl_snprintf (query, 1023,
-	       _("INSERT INTO %s VALUES (NULL,%s,\"%s\",\"%s\",\"%s\",\"%s\""),
-	       db_table,
-	       id >= 0 ? num : _("NULL"),
-	       db_entry->host,
-	       db_entry->time,
-	       db_entry->date,
-	       db_entry->sev);
-  */
-
   sl_snprintf (values, 1023, _("(%s,%c%s%c,%c%s%c,%c%s%c,%c%s%c"),
 	       id >= 0 ? num : _("NULL"),
 	       '\'', db_entry->host,'\'', 
@@ -583,11 +593,14 @@ long sh_database_entry (dbins * db_entry, long id)
   if (size > 1)   { *end   = ')'; ++end;   *end   = '\0'; }
   if (c_size > 1) { *c_end = ')'; ++c_end; *c_end = '\0'; }
 
+  if (db_table[0]    == '\0')
+    sl_strlcpy(db_table, _("log"),       64);
+
   sl_snprintf (query, 2047,
 	       _("INSERT INTO %s %s VALUES %s"),
 	       db_table, columns, values);
 
-  /* fprintf(stderr, "%s\n", query); */
+  /* fprintf(stderr, "%s\n", query); */ 
   status = sh_database_query (query, &the_id);
 
   SL_RETURN(the_id, _("sh_database_entry"));
