@@ -299,8 +299,16 @@ int read_mbytes(int timeout_val, char * path, char * nbuf, int nbytes)
   int fd2;
   struct  sigaction  def_act;
   struct  sigaction  new_act;
+  sigset_t           unblock;
+
   def_act.sa_handler = SIG_IGN;
+
   new_act.sa_handler = sh_entropy_alarmhandle;
+  sigemptyset( &new_act.sa_mask );       /* set an empty mask       */
+  new_act.sa_flags = 0;                  /* init sa_flags           */
+
+  sigemptyset(&unblock);
+  sigaddset  (&unblock, SIGALRM);
 
   SL_ENTER(_("read_mbytes"));
 
@@ -316,6 +324,9 @@ int read_mbytes(int timeout_val, char * path, char * nbuf, int nbytes)
 	   */
 	  if (setjmp(entropy_timeout) != 0)
 	    {
+	      alarm(0);
+	      sigaction (SIGALRM, &def_act, NULL);
+	      sigprocmask(SIG_UNBLOCK, &unblock, NULL);
 	      TPT((0,FIL__,__LINE__, _("msg=<read_mbytes: timeout>\n"))); 
 	      close (fd2);
 	      SL_RETURN(0, _("read_mbytes"));
@@ -324,7 +335,7 @@ int read_mbytes(int timeout_val, char * path, char * nbuf, int nbytes)
 	  /* timeout after 30 seconds
 	   */
 	  alarm(0);
-	  sigaction (SIGALRM, &new_act, NULL);
+	  sigaction (SIGALRM, &new_act, &def_act);
 	  alarm(timeout_val);
 
 	  m_count = 0;
