@@ -1119,9 +1119,55 @@ int sh_files_pushdir (int class, char * str_s)
   SL_RETURN((0), _("sh_files_pushdir"));
 }  
 
+struct sh_dirent {
+  char             * sh_d_name;
+  struct sh_dirent * next;
+};
+
+static void kill_sh_dirlist (struct sh_dirent * dirlist)
+{
+  struct sh_dirent * this;
+
+  while (dirlist)
+    {
+      this    = dirlist->next;
+      SH_FREE(dirlist->sh_d_name);
+      SH_FREE(dirlist);
+      dirlist = this;
+    }
+  return;
+}
+  
+static struct sh_dirent * addto_sh_dirlist (struct dirent * thisEntry, 
+					    struct sh_dirent * dirlist)
+{
+  struct sh_dirent * this;
+  int i;
+
+  if (thisEntry == NULL)
+    return dirlist;
+
+  i = sl_strlen(thisEntry->d_name);
+  if (i == 0)
+    return dirlist;
+  ++i;
+
+  this = SH_ALLOC(sizeof(struct sh_dirent));
+  if (!this)
+    return dirlist;
+
+  this->sh_d_name = SH_ALLOC(i);
+  sl_strlcpy(this->sh_d_name, thisEntry->d_name, i);
+
+  this->next = dirlist;
+  return this;
+}
 
 int sh_files_checkdir (int iclass, int idepth, char * iname)
 {
+  struct sh_dirent * dirlist = NULL;
+  struct sh_dirent * dirlist_orig = NULL;
+
   DIR *           thisDir = NULL;
   struct dirent * thisEntry;
   int             status;
@@ -1244,170 +1290,182 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
   theDir.TotalBytes  = 0;
   sl_strlcpy (theDir.DirPath, iname, PATH_MAX+1); 
 
-  BREAKEXIT(sh_derr);
-  if (0 == (rand() % 5))
-    (void) sh_derr();
 
   /* ---- read ----
    */
   do {
-    thisEntry = readdir (thisDir);
-
-    if (thisEntry != NULL) {
-      ++theDir.NumAll;
-
-      if (sl_strcmp (thisEntry->d_name, ".") == 0)
-	{ 
-	  ++theDir.NumDirs;
-	  continue;
-	}
-
-      if (sl_strcmp (thisEntry->d_name, "..") == 0)
+      thisEntry = readdir (thisDir);
+      if (thisEntry != NULL) 
 	{
-	  ++theDir.NumDirs;
-	  continue;
-	}
-
-      /* ---- Check the file. ---- 
-       */
-      tmpcat = SH_ALLOC(PATH_MAX+2);
-      sl_strlcpy(tmpcat, iname,                 PATH_MAX+2);
-      if (sl_strlen(tmpcat) > 1 || tmpcat[0] != '/')
-	sl_strlcat(tmpcat, "/",                   PATH_MAX+2);
-      sl_strlcat(tmpcat, thisEntry->d_name,     PATH_MAX+2);
-
-      dst_ptr       = fileList;
-      rdepth_next   = rdepth - 1;
-      class_next    = class;
-      checked_flag  = -1;
-      cchecked_flag = -1;
-
-      while (dst_ptr) 
-	{
-	  if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
-	    {
-	      class_next    = dst_ptr->class;
-	      checked_flag  = dst_ptr->checked;
-	      cchecked_flag = dst_ptr->childs_checked;
-	      break;
+	  ++theDir.NumAll;
+	  if (sl_strcmp (thisEntry->d_name, ".") == 0)
+	    { 
+	      ++theDir.NumDirs;
+	      continue;
 	    }
-	  dst_ptr = dst_ptr->next;
-	}
-
-      /* ---- Has been checked already. ----
-       */
-      if (checked_flag == S_TRUE && cchecked_flag == S_TRUE)
-	continue;
-	  
-      /* --- May be true, false, or not found. --- 
-       */
-      if (checked_flag == S_TRUE)
-	{
-	  /* -- need only the file type --
-	   */
-	  checkit = sh_unix_get_ftype(tmpcat);
-	}
-      else
-	{
-	  /* -- need to check the file itself --
-	   */
-	  if (dst_ptr && sh.flag.reportonce == S_TRUE)
-	    dummy = dst_ptr->reported;
-	  checkit = sh_files_filecheck (class_next, 
-					iname, 
-					thisEntry->d_name,
-					&dummy);
-	  if (dst_ptr && checked_flag == S_FALSE)
-	    dst_ptr->checked = S_TRUE;
-	  /* Thu Mar  7 15:09:40 CET 2002 Propagate the 'reported' flag
-	   */
-	  if (dst_ptr && sh.flag.reportonce == S_TRUE)
-	    dst_ptr->reported = dummy;
-	}
-	  
-      if      (checkit == SH_FILE_REGULAR)   
-	++theDir.NumRegular;
-
-      else if (checkit == SH_FILE_DIRECTORY) 
-	{
-	  ++theDir.NumDirs;
-	  if (rdepth_next >= 0 && cchecked_flag != S_TRUE) 
+	  if (sl_strcmp (thisEntry->d_name, "..") == 0)
 	    {
-	      rdepth_next = rdepth - 1;
-
-	      /* check whether the new directory is in the
-	       * list with a recursion depth already defined
-	       */
-	      checked_flag  = -1;
-	      cchecked_flag = -1;
-
-	      dst_ptr     = dirListOne;
-	      while (dst_ptr) 
-		{
-		  if (0 == sl_strcmp(dst_ptr->name, tmpcat))
-		    {
-		      TPT((0, FIL__, __LINE__, 
-			   _("msg=<%s -> recursion depth %d\n>"),
-			   dst_ptr->name, dst_ptr->rdepth));
-		      rdepth_next   = dst_ptr->rdepth;
-		      class_next    = dst_ptr->class;
-		      /* 28. Aug 2001 reversed
-		       */
-		      cchecked_flag = dst_ptr->childs_checked;
-		      checked_flag  = dst_ptr->checked;
-		      break;
-		    }
-		  dst_ptr = dst_ptr->next;
-		}
-
-	      if (checked_flag == -1)
-		{
-		  dst_ptr     = dirListTwo;
-		  while (dst_ptr) 
-		    {
-		      if (0 == sl_strcmp(dst_ptr->name, tmpcat))
-			{
-			  TPT((0, FIL__, __LINE__, 
-			       _("msg=<%s -> recursion depth %d\n>"),
-			       dst_ptr->name, dst_ptr->rdepth));
-			  rdepth_next   = dst_ptr->rdepth;
-			  class_next    = dst_ptr->class;
-			  /* 28. Aug 2001 reversed
-			   */
-			  cchecked_flag = dst_ptr->childs_checked;
-			  checked_flag  = dst_ptr->checked;
-			  break;
-			}
-		      dst_ptr = dst_ptr->next;
-		    }
-		}
-
-	      if (cchecked_flag == S_FALSE)
-		{
-		  sh_files_checkdir (class_next, rdepth_next, tmpcat);
-		  dst_ptr->childs_checked = S_TRUE;
-		}
-	      else if (checked_flag == -1)
-		sh_files_checkdir (class_next, rdepth_next, tmpcat);
-		    
+	      ++theDir.NumDirs;
+	      continue;
 	    }
+	  dirlist = addto_sh_dirlist (thisEntry, dirlist);
 	}
-
-      else if (checkit == SH_FILE_SYMLINK)   ++theDir.NumSymlinks;
-      else if (checkit == SH_FILE_FIFO)      ++theDir.NumFifos;
-      else if (checkit == SH_FILE_SOCKET)    ++theDir.NumSockets;
-      else if (checkit == SH_FILE_CDEV)      ++theDir.NumCDev;
-      else if (checkit == SH_FILE_BDEV)      ++theDir.NumBDev;
-
-      SH_FREE(tmpcat);
-
-    }
-
-    if (sig_terminate == 1)
-      SL_RETURN((0), _("sh_files_checkdir"));
-
   } while (thisEntry != NULL);
 
+  closedir (thisDir);
+
+  dirlist_orig = dirlist;
+
+  do {
+
+    /* If the directory is empty, dirlist = NULL
+     */
+    if (!dirlist)
+      break;
+
+    BREAKEXIT(sh_derr);
+    if (0 == (rand() % 5))
+      (void) sh_derr();
+    
+    /* ---- Check the file. ---- 
+     */
+    tmpcat = SH_ALLOC(PATH_MAX+2);
+    sl_strlcpy(tmpcat, iname,                   PATH_MAX+2);
+    if (sl_strlen(tmpcat) > 1 || tmpcat[0] != '/')
+      sl_strlcat(tmpcat, "/",                   PATH_MAX+2);
+    sl_strlcat(tmpcat, dirlist->sh_d_name,      PATH_MAX+2);
+    
+    dst_ptr       = fileList;
+    rdepth_next   = rdepth - 1;
+    class_next    = class;
+    checked_flag  = -1;
+    cchecked_flag = -1;
+    
+    while (dst_ptr) 
+      {
+	if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
+	  {
+	    class_next    = dst_ptr->class;
+	    checked_flag  = dst_ptr->checked;
+	    cchecked_flag = dst_ptr->childs_checked;
+	    break;
+	  }
+	dst_ptr = dst_ptr->next;
+      }
+    
+    /* ---- Has been checked already. ----
+     */
+    if (checked_flag == S_TRUE && cchecked_flag == S_TRUE)
+      continue;
+    
+    /* --- May be true, false, or not found. --- 
+     */
+    if (checked_flag == S_TRUE)
+      {
+	/* -- need only the file type --
+	 */
+	checkit = sh_unix_get_ftype(tmpcat);
+      }
+    else
+      {
+	/* -- need to check the file itself --
+	 */
+	if (dst_ptr && sh.flag.reportonce == S_TRUE)
+	  dummy = dst_ptr->reported;
+	checkit = sh_files_filecheck (class_next, 
+				      iname, 
+				      dirlist->sh_d_name,
+				      &dummy);
+	if (dst_ptr && checked_flag == S_FALSE)
+	  dst_ptr->checked = S_TRUE;
+	/* Thu Mar  7 15:09:40 CET 2002 Propagate the 'reported' flag
+	 */
+	if (dst_ptr && sh.flag.reportonce == S_TRUE)
+	  dst_ptr->reported = dummy;
+      }
+    
+    if      (checkit == SH_FILE_REGULAR)   
+      ++theDir.NumRegular;
+    
+    else if (checkit == SH_FILE_DIRECTORY) 
+      {
+	++theDir.NumDirs;
+	if (rdepth_next >= 0 && cchecked_flag != S_TRUE) 
+	  {
+	    rdepth_next = rdepth - 1;
+	    
+	    /* check whether the new directory is in the
+	     * list with a recursion depth already defined
+	     */
+	    checked_flag  = -1;
+	    cchecked_flag = -1;
+	    
+	    dst_ptr     = dirListOne;
+	    while (dst_ptr) 
+	      {
+		if (0 == sl_strcmp(dst_ptr->name, tmpcat))
+		  {
+		    TPT((0, FIL__, __LINE__, 
+			 _("msg=<%s -> recursion depth %d\n>"),
+			 dst_ptr->name, dst_ptr->rdepth));
+		    rdepth_next   = dst_ptr->rdepth;
+		    class_next    = dst_ptr->class;
+		    /* 28. Aug 2001 reversed
+		     */
+		    cchecked_flag = dst_ptr->childs_checked;
+		    checked_flag  = dst_ptr->checked;
+		    break;
+		  }
+		dst_ptr = dst_ptr->next;
+	      }
+	    
+	    if (checked_flag == -1)
+	      {
+		dst_ptr     = dirListTwo;
+		while (dst_ptr) 
+		  {
+		    if (0 == sl_strcmp(dst_ptr->name, tmpcat))
+		      {
+			TPT((0, FIL__, __LINE__, 
+			     _("msg=<%s -> recursion depth %d\n>"),
+			     dst_ptr->name, dst_ptr->rdepth));
+			rdepth_next   = dst_ptr->rdepth;
+			class_next    = dst_ptr->class;
+			/* 28. Aug 2001 reversed
+			 */
+			cchecked_flag = dst_ptr->childs_checked;
+			checked_flag  = dst_ptr->checked;
+			break;
+		      }
+		    dst_ptr = dst_ptr->next;
+		  }
+	      }
+	    
+	    if (cchecked_flag == S_FALSE)
+	      {
+		sh_files_checkdir (class_next, rdepth_next, tmpcat);
+		dst_ptr->childs_checked = S_TRUE;
+	      }
+	    else if (checked_flag == -1)
+	      sh_files_checkdir (class_next, rdepth_next, tmpcat);
+	    
+	  }
+      }
+    
+    else if (checkit == SH_FILE_SYMLINK)   ++theDir.NumSymlinks;
+    else if (checkit == SH_FILE_FIFO)      ++theDir.NumFifos;
+    else if (checkit == SH_FILE_SOCKET)    ++theDir.NumSockets;
+    else if (checkit == SH_FILE_CDEV)      ++theDir.NumCDev;
+    else if (checkit == SH_FILE_BDEV)      ++theDir.NumBDev;
+    
+    SH_FREE(tmpcat);
+    
+    if (sig_terminate == 1)
+      SL_RETURN((0), _("sh_files_checkdir"));
+    
+    dirlist = dirlist->next;
+    
+  } while (dirlist != NULL);
 
   sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_DSUM,
 		   theDir.NumDirs,
@@ -1417,9 +1475,9 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
 		   theDir.NumSockets,
 		   theDir.NumCDev,
 		   theDir.NumBDev);
-
   
-  closedir (thisDir);
+  kill_sh_dirlist (dirlist_orig);
+
   SH_FREE(tmpname);
   SL_RETURN((0), _("sh_files_checkdir"));
 }
@@ -1669,10 +1727,51 @@ int sh_files_test_setup_int (dirstack_t * pdirL)
   SL_RETURN((0), _("sh_files_test_setup"));
 }
       
+int sh_files_test_double (dirstack_t * firstList, dirstack_t * secondList)
+{
+  int          count;
+
+  dirstack_t * saveList = firstList;
+  dirstack_t * tempList;
+
+  while (firstList)
+    {
+      tempList = saveList;
+      count    = 0;
+
+      while (tempList)
+	{
+	  if (0 == sl_strcmp(firstList->name, tempList->name))
+	    ++count;
+	  tempList = tempList->next;
+	}
+      tempList = secondList;
+      while (tempList)
+	{
+	  if (0 == sl_strcmp(firstList->name, tempList->name))
+	    ++count;
+	  tempList = tempList->next;
+	}
+      if (count > 1)
+	sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_DOUBLE,
+			 firstList->name);
+      firstList = firstList->next;
+    }
+  return 0;
+}
+      
 int sh_files_test_setup ()
 {
+  /* Test for modifications allowed in ReadOnly directory
+   */  
   sh_files_test_setup_int (dirListOne);
   sh_files_test_setup_int (dirListTwo);
+
+  /* Test for files/dirz defined twice
+   */  
+  sh_files_test_double (dirListOne, dirListTwo);
+  sh_files_test_double (dirListTwo, dirListOne);
+  sh_files_test_double (fileList,   NULL);
   return 0;
 }
 
