@@ -1,5 +1,5 @@
 /* SAMHAIN file system integrity testing                                   */
-/* Copyright (C) 1999, 2000 Rainer Wichmann                                */
+/* Copyright (C) 1999, 2000, 2001, 2002 Rainer Wichmann                    */
 /*                                                                         */
 /*  This program is free software; you can redistribute it                 */
 /*  and/or modify                                                          */
@@ -1343,7 +1343,7 @@ sh_file_t * sh_hash_push_int (file_type * buf, char * fileHash)
 #endif
 
 #ifndef PRIi64
-#define PRIi64 "ld"
+#define PRIi64 "lld"
 #endif
 
 char * my_size_format()
@@ -2052,6 +2052,68 @@ int hash_remove_tree (char * s)
 #endif
 #endif
 
+static int ListFullDetail = S_FALSE;
+
+int set_full_detail (char * c)
+{
+  ListFullDetail = S_TRUE;
+  /* warning: unused parameter `c' */
+  if (c)
+    return 0;
+  else
+    return 0;
+}
+ 
+void sh_hash_list_db_entry_full_detail (sh_file_t * p)
+{
+  char * tmp;
+  char   str[81];
+
+  printf(_("%7ld %10s %5d %12s %5d %3d %-8s %5d %-8s %5d "),
+	 (unsigned long) p->theFile.ino,
+	 p->theFile.c_mode, (int) p->theFile.mode,
+	 p->theFile.c_attributes, (int) p->theFile.attributes,
+	 (int) p->theFile.hardlinks,
+	 p->theFile.c_owner, (int) p->theFile.owner, 
+	 p->theFile.c_group, (int) p->theFile.group);
+#ifdef HOST_IS_AIX
+  /* AIX has a completely different definition (%lli instead of "lli") 
+   */
+  if ('c' == p->theFile.c_mode[0] || 'b' == p->theFile.c_mode[0])
+    snprintf(str, 80, "PRIi64", p->theFile.rdev);
+  else
+    snprintf(str, 80, "PRIi64", p->theFile.size);
+#else
+  /* not AIX
+   */
+  if ('c' == p->theFile.c_mode[0] || 'b' == p->theFile.c_mode[0])
+    snprintf(str, 80, "%"PRIi64, p->theFile.rdev);
+  else
+    snprintf(str, 80, "%"PRIi64, p->theFile.size);
+#endif
+  printf( _(" %8s"), str);
+
+  printf( _(" %s"), sh_unix_gmttime (p->theFile.ctime));
+  printf( _(" %s"), sh_unix_gmttime (p->theFile.mtime));
+  printf( _(" %s"), sh_unix_gmttime (p->theFile.atime));
+  printf( _(" %s"), p->theFile.checksum);
+
+  tmp = sh_util_safe_name(p->fullpath);
+  printf( _(" %s"), tmp);
+  SH_FREE(tmp);
+
+  if ('l' == p->theFile.c_mode[0])
+    {
+      tmp = sh_util_safe_name(p->linkpath);
+      printf(_(" -> %s\n"), tmp);
+      SH_FREE(tmp);
+    }
+  else
+    printf("\n");
+
+  return;
+}
+
 void sh_hash_list_db_entry (sh_file_t * p)
 {
   char nowtime[128];
@@ -2085,7 +2147,7 @@ void sh_hash_list_db_entry (sh_file_t * p)
   if ('l' == p->theFile.c_mode[0])
     {
       tmp = sh_util_safe_name(p->linkpath);
-      printf(_(" -> %s\n"), p->linkpath);
+      printf(_(" -> %s\n"), tmp);
       SH_FREE(tmp);
     }
   else
@@ -2136,7 +2198,12 @@ int sh_hash_list_db (char * db_file)
     {
       p = sh_hash_getdataent (fd, line);
       if (p != NULL)
-	sh_hash_list_db_entry (p); 
+	{
+	  if (ListFullDetail == S_FALSE)
+	    sh_hash_list_db_entry (p); 
+	  else
+	    sh_hash_list_db_entry_full_detail (p); 
+	}
       else
 	{
 	  /* fprintf(stderr, "FIXME: p == NULL\n"); */
@@ -2147,6 +2214,9 @@ int sh_hash_list_db (char * db_file)
   if (line != NULL)
     SH_FREE(line);
   sl_close (fd);
+
+  fflush(NULL);
+
   _exit(EXIT_SUCCESS);
   return 0; 
 }
