@@ -907,8 +907,9 @@ static FILE * sh_mail_start_conn (int aFlag)
 
   if (fd < 0)
     {
-      sh_error_handle((-1), FIL__, __LINE__, error_num, MSG_TCP_NETRP, 
-		      error_msg, (long) IPPORT_SMTP, error_call);
+      sh_error_handle ((-1), FIL__, __LINE__, error_num, 
+		       MSG_E_NET, error_msg, error_call,
+		       _("email"), m_machine);
       SL_RETURN( NULL, _("sh_mail_start_conn"));
     }
 
