@@ -45,6 +45,8 @@
 #include "sh_database.h"
 #endif
 
+extern int set_reverse_lookup (char * c);
+
 #undef  FIL__
 #define FIL__  _("sh_readconf.c")
 
@@ -825,6 +827,8 @@ cfg_options ext_table[] = {
   sh_forward_use_clt_class },
   { N_("severitylookup"),      SH_SECTION_SRV,  SH_SECTION_MISC, 
   sh_forward_lookup_level },
+  { N_("setclientfromaccept"), SH_SECTION_SRV,  SH_SECTION_MISC, 
+    set_socket_peer },
   { N_("client"),              SH_SECTION_CLIENTS,           SH_SECTION_NONE, 
     sh_forward_register_client },
 #endif
@@ -902,6 +906,9 @@ cfg_options ext_table[] = {
     enable_msgq },
 #endif
 
+  { N_("setreverselookup"),    SH_SECTION_MISC,  SH_SECTION_NONE, 
+    set_reverse_lookup },
+
   { N_("setdatabasepath"),    SH_SECTION_MISC,  SH_SECTION_NONE, 
     sh_readconf_set_database_path },
 
