@@ -491,7 +491,7 @@ int sh_suidchk_set_activate (char * c)
   SL_RETURN(i, _("sh_suidchk_set_activate"));
 }
 
-#ifdef FSTYPE_STATFS
+#if defined(FSTYPE_STATFS) || defined(FSTYPE_AIX_STATFS)
 /* dirname.c -- return all but the last element in a path
    Copyright (C) 1990 Free Software Foundation, Inc.
 
@@ -514,16 +514,14 @@ int sh_suidchk_set_activate (char * c)
    Assumes that trailing slashes have already been
    removed.  */
 
-char *
-dirname (path)
-     char *path;
+char * sh_dirname (const char * path)
 {
   char *newpath;
   char *slash;
   int length;                   /* Length of result, not including NUL.  */
 
   slash = strrchr (path, '/');
-  if (slash == 0)
+  if (slash == NULL)
     {
       /* File is in the current directory.  */
       path = ".";
@@ -538,10 +536,10 @@ dirname (path)
       length = slash - path + 1;
     }
   newpath = (char *) SH_ALLOC (length + 1);
-  if (newpath == 0)
-    return 0;
+  if (newpath == NULL)
+    return NULL;
   strncpy (newpath, path, length);
-  newpath[length] = 0;
+  newpath[length] = '\0';
   return newpath;
 }
 /* #ifdef FSTYPE_STATFS */
@@ -763,9 +761,7 @@ fstype_to_string (t)
   switch (t)
     {
     case MNT_AIX:
-#if 0				/* NFS filesystems are actually MNT_AIX. */
-      return "aix";
-#endif
+      return _("aix");	/* NFS filesystems are actually MNT_AIX. */
     case MNT_NFS:
       return _("nfs");
     case MNT_JFS:
@@ -917,7 +913,7 @@ filesystem_type_uncached (path, relpath, statp)
 	  if (NULL == hasmntopt(mnt, "nosuid"))
 	    type = mnt->mnt_type;
 	  else
-	    type = "nosuid";
+	    type = _("nosuid");
 	}
     }
 
@@ -990,7 +986,7 @@ filesystem_type_uncached (path, relpath, statp)
        /* patch by Konstantin Khrooschev <nathoo@co.ru> 
 	*/
        if( fss.f_flag & ST_NOSUID )
-         type = "nosuid";
+         type = _("nosuid");
     }
 #endif
 
@@ -1000,13 +996,13 @@ filesystem_type_uncached (path, relpath, statp)
 #ifdef MNT_VISFLAGMASK
   int flags;
 #endif
-  /* extern char * dirname(const char *path); */
+  /* char * sh_dirname(const char *path); */
 
   if (path == NULL || relpath == NULL)
     return NULL;
 
   if (S_ISLNK (statp->st_mode))
-    p = dirname (relpath);
+    p = sh_dirname (relpath);
   else
     p = relpath;
 
@@ -1034,13 +1030,16 @@ filesystem_type_uncached (path, relpath, statp)
 #else
       type = fstype_to_string (fss.f_type);
 #endif
+
+#ifdef HAVE_STRUCT_STATFS_F_FLAGS
 #ifdef MNT_VISFLAGMASK
       flags = fss.f_flags & MNT_VISFLAGMASK;
       if (flags & MNT_NOSUID)
 #else 
       if (fss.f_flags & MNT_NOSUID) 
 #endif
-         type = "nosuid";
+         type = _("nosuid");
+#endif
     }
   if (p != relpath)
     SH_FREE (p);
