@@ -195,21 +195,21 @@ char * sh_tools_errmessage (int tellme)
 
 #ifdef HOST_NOT_FOUND
     if (tellme == HOST_NOT_FOUND)  
-      return _("The specified host is unknown.");
+      return _("The specified host is unknown: ");
 #endif
 #ifdef NO_ADDRESS
     if (tellme == NO_ADDRESS)  
-      return _("The requested name is valid but does not have an IP address.");
+      return _("The requested name is valid but does not have an IP address: ");
 #endif
 #ifdef NO_RECOVERY
     if (tellme == NO_RECOVERY)  
-      return _("A non-recoverable name server error occurred.");
+      return _("A non-recoverable name server error occurred: ");
 #endif
 #ifdef TRY_AGAIN
     if (tellme == TRY_AGAIN)  
-      return _("A temporary error occurred on an authoritative name server.The specified host is unknown.");
+      return _("A temporary error occurred on an authoritative name server. The specified host is unknown: ");
 #endif
-    return _("Unknown error.");
+    return _("Unknown error");
 }
 
 int is_numeric (const char * address)
@@ -276,12 +276,19 @@ void delete_cache()
   SL_RET0(_("delete_cache"));
 }
       
+int DoReverseLookup = S_TRUE;
+
+int set_reverse_lookup (char * c)
+{
+  return sh_util_flagval(c, &DoReverseLookup);
+}
+
 int connect_port (char * address, int port, 
 		  char * ecall, int * errnum, char * errmsg, int errsiz)
 {
   struct in_addr       haddr;   /* host address from numeric                */
                                 /* host details returned by the DNS         */
-  struct hostent *host_entry=NULL;   
+  struct hostent *host_entry = NULL;   
   struct sockaddr_in sinr;      /* socket to the remote host                */
 
   char   * host_name;
@@ -333,7 +340,9 @@ int connect_port (char * address, int port,
 #endif
 	{
       
-	  if ((host_entry = gethostbyname(address)) == NULL) 
+	  host_entry = gethostbyname(address);
+
+	  if (host_entry == NULL || host_entry->h_addr == NULL) 
 	    {
 	      sl_strlcpy(ecall, _("gethostbyname"), SH_MINIBUF);
 #ifndef HOST_IS_HPUX
@@ -342,6 +351,7 @@ int connect_port (char * address, int port,
 	      *errnum = 666;
 #endif
 	      sl_strlcpy(errmsg, sh_tools_errmessage (*errnum), errsiz);
+	      sl_strlcat(errmsg, address, errsiz); 
 	      fail = (-1);
 	    }
 	  else
@@ -353,41 +363,63 @@ int connect_port (char * address, int port,
 
 	      /* reverse DNS lookup
 	       */
-
-	      host_name = SH_ALLOC(sl_strlen(host_entry->h_name) + 1);
-	      if (sl_strlen(host_entry->h_name) > 0)
-		strcpy(host_name, host_entry->h_name);
-	      else
-		host_name[0] = '\0';
-
-	      host_entry = gethostbyaddr ((char *) &sinr.sin_addr, 
-					  sizeof(struct in_addr),
-					  AF_INET);
-	      if (host_entry == NULL)
+	      if (DoReverseLookup == S_TRUE)
 		{
-		  sl_strlcpy(ecall, _("gethostbyaddr"), SH_MINIBUF);
-#ifndef NO_H_ERRNO
-		  *errnum = h_errno;
-#else
-		  *errnum = 666;
-#endif
-		  sl_strlcpy(errmsg, sh_tools_errmessage (*errnum), errsiz);
-		  fail = (-1);
-		}
-
-	      if ((*errnum = sl_strcmp (host_name, host_entry->h_name)) != 0 || 
-		  sl_strlen(host_entry->h_name) == 0)
-		{ 
-		  if (*errnum)
-		    sl_strlcpy(ecall, _("strcmp"), SH_MINIBUF);
+		  if (host_entry->h_name == NULL)
+		    {
+		      host_name = SH_ALLOC(1);
+		      host_name[0] = '\0';
+		    }
 		  else
-		    sl_strlcpy(ecall, _("strlen"), SH_MINIBUF);
-		  sl_strlcpy(errmsg, _("Reverse lookup failed."), errsiz);
-		  fail = -1;
+		    {
+		      host_name = SH_ALLOC(sl_strlen(host_entry->h_name) + 1);
+		      if (sl_strlen(host_entry->h_name) > 0)
+			strcpy(host_name, host_entry->h_name);
+		      else
+			host_name[0] = '\0';
+		    }
+
+		  host_entry = gethostbyaddr ((char *) &sinr.sin_addr, 
+					      sizeof(struct in_addr),
+					      AF_INET);
+		  if (host_entry == NULL || host_entry->h_name == NULL)
+		    {
+		      sl_strlcpy(ecall, _("gethostbyaddr"), SH_MINIBUF);
+#ifndef NO_H_ERRNO
+		      *errnum = h_errno;
+#else
+		      *errnum = 666;
+#endif
+		      sl_strlcpy(errmsg, 
+				 sh_tools_errmessage (*errnum), errsiz);
+		      sl_strlcat(errmsg, 
+				 inet_ntoa (*(struct in_addr *) &(sinr.sin_addr)),
+				 errsiz); 
+		      fail = (-1);
+		    }
+		  else
+		    {
+		      *errnum = 0;
+		      if (sl_strlen(host_entry->h_name) == 0 || 
+			  (*errnum = sl_strcmp(host_name,host_entry->h_name)) != 0)
+			{ 
+			  if (*errnum)
+			    sl_strlcpy(ecall, _("strcmp"), SH_MINIBUF);
+			  else
+			    sl_strlcpy(ecall, _("strlen"), SH_MINIBUF);
+			  sl_strlcpy(errmsg, _("Reverse lookup failed."), 
+				     errsiz);
+			  sl_strlcat(errmsg, address, errsiz);
+			  sl_strlcat(errmsg, _(" vs "), errsiz);
+			  sl_strlcat(errmsg, 
+				     inet_ntoa (*(struct in_addr *) &(sinr.sin_addr)),
+				     errsiz);
+			  fail = -1;
+			}
+		    }
+
+		  SH_FREE(host_name);
 		}
-
-	      SH_FREE(host_name);
-
 	    }
 	}
   
@@ -435,17 +467,22 @@ int connect_port (char * address, int port,
 	sl_strlcpy(ecall, _("socket"), SH_MINIBUF);
 	*errnum = status;
 	sl_strlcpy(errmsg, sh_error_message (status), errsiz);
+	sl_strlcat(errmsg, _(", address "), errsiz);
+	sl_strlcat(errmsg, address, errsiz);
       }
     }
   
   if (fail != (-1)) {
     
-    if ( retry_connect(FIL__, __LINE__, fd, (struct sockaddr *) &sinr, sizeof(sinr)) < 0) 
+    if ( retry_connect(FIL__, __LINE__, fd, 
+		       (struct sockaddr *) &sinr, sizeof(sinr)) < 0) 
       {
 	status = errno;
 	sl_strlcpy(ecall, _("connect"), SH_MINIBUF);
 	*errnum = status;
 	sl_strlcpy(errmsg, sh_error_message (status), errsiz);
+	sl_strlcat(errmsg, _(", address "), errsiz);
+	sl_strlcat(errmsg, address, errsiz);
 	close(fd);
 	fail = (-1); 
       }
