@@ -2251,25 +2251,31 @@ int sh_unix_getinfo (int level,
   /* --- Owner and group. ---
    */
 
-  if ( (name = sh_unix_getGIDname(level, buf.st_gid)) != NULL) {
-    sl_strlcpy (theFile->c_group, name, GROUP_MAX);
+  if ( (name = sh_unix_getGIDname(ShDFLevel[SH_ERR_T_NAME], buf.st_gid)) != NULL) {
+    sl_strlcpy (theFile->c_group, name, GROUP_MAX+1);
   } else {
     tmp2 = sh_util_safe_name (theFile->fullpath);
-    sh_error_handle (level, FIL__, __LINE__, ENOENT, MSG_FI_NOGRP,
+    /* level -> ShDFLevel[SH_ERR_T_NAME] */
+    sh_error_handle (ShDFLevel[SH_ERR_T_NAME], FIL__, __LINE__, ENOENT, 
+		     MSG_FI_NOGRP,
 		     (long) buf.st_gid, tmp2);
     SH_FREE(tmp2);
-    theFile->c_group[0] = '\0';
+    sl_snprintf(theFile->c_group, GROUP_MAX+1, "%d", (long) buf.st_gid); 
+    /* theFile->c_group[0] = '\0'; */
   }
 
   
-  if ( (name = sh_unix_getUIDname(level, buf.st_uid)) != NULL) {
+  if ( (name = sh_unix_getUIDname(ShDFLevel[SH_ERR_T_NAME], buf.st_uid)) != NULL) {
     sl_strlcpy (theFile->c_owner, name, USER_MAX+1);
   } else {
     tmp2 = sh_util_safe_name (theFile->fullpath);
-    sh_error_handle (level, FIL__, __LINE__, ENOENT, MSG_FI_NOUSR,
+    /* level -> ShDFLevel[SH_ERR_T_NAME] */
+    sh_error_handle (ShDFLevel[SH_ERR_T_NAME], FIL__, __LINE__, ENOENT, 
+		     MSG_FI_NOUSR,
 		     (long) buf.st_uid, tmp2);
     SH_FREE(tmp2);
-    theFile->c_owner[0] = '\0';
+    sl_snprintf(theFile->c_owner, USER_MAX+1, "%d", (long) buf.st_uid); 
+    /* theFile->c_owner[0] = '\0'; */
   }
 
   /* --- Output the file. ---
