@@ -60,6 +60,9 @@ void dbg_use_console()
   return;
 }
 
+extern char *  sh_log_console_name ();
+
+
 void dbg(int mod_ebene, char * file, int line, char * fmt, ...)
 {
   FILE * dfile;
@@ -102,7 +105,7 @@ void dbg(int mod_ebene, char * file, int line, char * fmt, ...)
 	    }
 	  else
 	    {
-	      dfile = fopen(_("/dev/console"), "w");
+	      dfile = fopen(sh_log_console_name(), "w");
 	      fprintf (dfile, _("DBG: file=<%s>, line=<%d>, "), file, line);
 	      va_start(ap, fmt);
 	      vfprintf  (dfile, fmt, ap);
@@ -118,7 +121,7 @@ void dbg(int mod_ebene, char * file, int line, char * fmt, ...)
 		   file, line, stein);
 	else
 	  {
-	    dfile = fopen(_("/dev/console"), "w");
+	    dfile = fopen(sh_log_console_name(), "w");
 	    fprintf (dfile, 
 		     _("MILEST: file=<%s>, line=<%d>, milestone=<%d>, "), 
 		     file, line, stein);
@@ -141,7 +144,7 @@ void dbg(int mod_ebene, char * file, int line, char * fmt, ...)
 	    }
 	  else
 	    {
-	      dfile = fopen(_("/dev/console"), "w");
+	      dfile = fopen(sh_log_console_name(), "w");
 	      fprintf (dfile, 
 		       _("%sTRACE : file=<%s>, line=<%d>, "),
 		       indent_it, file, line);
@@ -1104,11 +1107,20 @@ int sl_policy_get_real(char * user)
   SL_REQUIRE(uids_are_stored == SL_FALSE, _("uids_are_stored == SL_FALSE"));
   SL_REQUIRE (sl_save_uids() == SL_ENONE, _("sl_save_uids() == SL_ENONE"));
 
-  tempres = getpwnam(user);
-  SL_REQUIRE (NULL != tempres, _("tempres != NULL"));
+  if (euid == 0 || ruid == 0)
+    {
+      tempres = getpwnam(user);
+      fprintf(stderr, "user = %s", user);
+      SL_REQUIRE (NULL != tempres, _("tempres != NULL"));
   
-  rgid_orig = tempres->pw_gid;
-  ruid_orig = tempres->pw_uid;
+      rgid_orig = tempres->pw_gid;
+      ruid_orig = tempres->pw_uid;
+    }
+  else
+    {
+      rgid_orig = rgid;
+      ruid_orig = ruid;
+    }
 
   SL_REQUIRE (sl_drop_privileges() == SL_ENONE,
 	      _("sl_drop_privileges() == SL_ENONE"));
@@ -1665,6 +1677,7 @@ int sl_forward (SL_TICKET ticket)
   SL_IRETURN(SL_ENONE, _("sl_forward")); 
 }
 
+
 int sl_sync (SL_TICKET ticket) 
 {
   register int fd;
