@@ -1,8 +1,12 @@
+/* debug problems        */
+/* #define TRUST_DEBUG   */
+
 /* switch off full check */
 /* #define TEST_ONLY     */
 
-/* debug problems        */
-/* #define TRUST_DEBUG   */
+/* standalone            */
+/* #define TRUST_MAIN    */
+/* $(CC) -DTRUST_MAIN -DSL_ALWAYS_TRUSTED=...  */
 
 /* LINTLIBRARY */
 /*
@@ -76,9 +80,10 @@
  * is always copied between buffers of length MAXFILENAME.
  */
 
+#ifndef  TRUST_MAIN
 #include "config_xor.h"
 #include "sh_calls.h"
-
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -89,18 +94,34 @@
 #include <string.h>
 #include <unistd.h>
 
-/* -- Defined in config.h. --
-
-#ifndef _(string)
-#define _(string) string
-#endif
-
-#ifndef N_(string)
-#define N_(string) string
-#endif
-*/
 
+#ifndef TRUST_MAIN
 #include "slib.h"
+#else
+
+#define TRUST_DEBUG
+#define SL_FALSE 0
+#define SL_TRUE  1
+#define SL_ENTER(string)
+#define SL_IRETURN(a, b)  return a
+#define retry_lstat(a,b,c,d) lstat(c,d)
+#define _(string)  string 
+#define N_(string) string
+#define MAXFILENAME     2048
+static int sl_errno = 0;
+#define SL_ENONE         0
+#define SL_ENULL     -1024     /* Invalid use of NULL pointer.         */
+#define SL_ERANGE    -1025     /* Argument out of range.               */
+#define SL_ETRUNC    -1026     /* Result truncated.                    */
+#define SL_EINTERNAL -1028     /* Internal error.                      */
+#define SL_EBADFILE  -1030     /* File access error. Check errno.      */
+#define SL_EBADNAME  -1040     /* Invalid name.                        */
+#define SL_ESTAT     -1041     /* stat of file failed. Check errno.    */
+#define SL_EBADUID   -1050	/* Owner not trustworthy.              */
+#define SL_EBADGID   -1051	/* Group writeable and not trustworthy.*/
+#define SL_EBADOTH   -1052	/* World writeable.                    */
+
+#endif
 
 
 #if defined(__linux__)
@@ -119,12 +140,20 @@
  */
 
 
+#ifndef TRUST_MAIN
+
 #if defined(HAVE_GETCWD) && !defined(HAVE_BROKEN_GETCWD)
 #define CURDIR(buf,nbuf)	getcwd((buf), (nbuf))
 #elif defined(HAVE_GETWD)
 #define CURDIR(buf,nbuf)	getwd((buf))
 #endif
 
+#else
+
+#define CURDIR(buf,nbuf)	getcwd((buf), (nbuf))
+
+#endif
+
 
 
 /*
@@ -267,8 +296,10 @@ int dirz(char *path)
 static 
 int getfname(char *fname, char *rbuf, int rsz)
 {
+#ifndef TRUST_MAIN
   register int status;
-  
+#endif
+
   SL_ENTER(_("getfname"));
   /*
    * do the initial checking
@@ -286,7 +317,12 @@ int getfname(char *fname, char *rbuf, int rsz)
   else
     {
       if (CURDIR(rbuf, rsz)  == NULL)
-	SL_IRETURN(SL_EBADNAME, _("getfname"));
+	{
+#ifdef TRUST_DEBUG
+	  fprintf(stderr, "trustfile: getcwd failed\n");
+#endif 
+	  SL_IRETURN(SL_EBADNAME, _("getfname"));
+	}
     }
   
   /*
@@ -294,11 +330,16 @@ int getfname(char *fname, char *rbuf, int rsz)
    */
   if (fname != NULL && *fname != '\0')
     {
+#ifndef TRUST_MAIN
       status = sl_strlcat(rbuf, "/", rsz);
       if (status == SL_ENONE)
 	status = sl_strlcat(rbuf, fname, rsz);
       if (status != SL_ENONE)
 	SL_IRETURN(status, _("getfname"));
+#else
+      strncat(rbuf, "/",   rsz-strlen(rbuf));
+      strncat(rbuf, fname, rsz-strlen(rbuf));
+#endif
     }
   SL_IRETURN(dirz(rbuf), _("getfname"));
 }
@@ -313,14 +354,28 @@ int isin(long n, long *list)
   while(*list != -1 && *list != n)
     {
 #ifdef TRUST_DEBUG
-      fprintf (stderr, "FIXME: n=%ld, *list=%ld\n", n, *list);
+      fprintf (stderr, 
+	       "trustfile: owner_uid=%ld, trusted uid=%ld, no match\n", 
+	       n, *list);
 #endif 
       list++;
     }
 
   if (*list == -1)
-    SL_IRETURN(SL_FALSE, _("isin"));
+    {
+#ifdef TRUST_DEBUG
+      fprintf (stderr, 
+	       "trustfile: owner_uid=%ld, no match with any trusted user --> ERROR\n", 
+	       n);
+#endif 
+      SL_IRETURN(SL_FALSE, _("isin"));
+    }
 
+#ifdef TRUST_DEBUG
+  fprintf (stderr, 
+	   "trustfile: owner_uid=%ld, trusted_uid=%ld, match found --> OK\n", 
+	   n, *list);
+#endif 
   SL_IRETURN(SL_TRUE, _("isin"));
 }
 
@@ -378,11 +433,17 @@ int onlytrustedingrp(long grp, long *ulist)
   SL_ENTER(_("onlytrustedingrp"));
 
 #ifdef TRUST_DEBUG
-      fprintf(stderr, "GROUP: %ld\n", grp); 
+      fprintf(stderr, "trustfile: group writeable, group_gid: %ld\n", grp); 
 #endif
 
   if ((g = getgrgid(grp)) == NULL)
-    SL_IRETURN(SL_FALSE, _("onlytrustedingrp") );
+    {
+#ifdef TRUST_DEBUG
+      fprintf(stderr, "trustfile: group_gid: %ld, no such group --> ERROR\n", 
+	      grp); 
+#endif
+      SL_IRETURN(SL_FALSE, _("onlytrustedingrp") );
+    }
 
   /* empty group -> no problem
    
@@ -396,7 +457,7 @@ int onlytrustedingrp(long grp, long *ulist)
     {
       flag = -1;
 #ifdef TRUST_DEBUG
-      fprintf(stderr, "GROUP MEMBER: %s\n", *p); 
+      fprintf(stderr, "trustfile: group_member: %s\n", *p); 
 #endif
       /* map user name to UID and compare 
        */
@@ -407,16 +468,44 @@ int onlytrustedingrp(long grp, long *ulist)
 	}
       else              /* check list of trusted users */
 	{
+#ifdef TRUST_DEBUG
+	  fprintf (stderr, 
+		   "trustfile: uid=%ld, checking whether it is trusted\n",
+		   (long)(w->pw_uid));
+#endif 
 	  for(u = ulist; *u != -1; u++)
 	    {
 	      if (*u == (long)(w->pw_uid) )
-		flag = 0;
+		{
+#ifdef TRUST_DEBUG
+		  fprintf (stderr, 
+			   "trustfile: uid=%ld, trusted_uid=%ld, match found --> OK\n", 
+			   (long)(w->pw_uid), *u);
+#endif 
+		  flag = 0;
+		  break;
+		}
+	      else
+		{
+#ifdef TRUST_DEBUG
+		  fprintf (stderr, 
+			   "trustfile: uid=%ld, trusted_uid=%ld, no match\n", 
+			   (long)(w->pw_uid), *u);
+#endif 
+		  ;
+		}
 	    }
 	}
       /* not found 
        */
       if (flag == -1)
+	{
+#ifdef TRUST_DEBUG
+	  fprintf (stderr, 
+		   "trustfile: user=%s (gid %ld), not a trusted user --> ERROR\n", *p, grp);
+#endif 
 	  SL_IRETURN(SL_FALSE, _("onlytrustedingrp"));
+	}
     }
 
 #ifndef TEST_ONLY	
@@ -428,7 +517,8 @@ int onlytrustedingrp(long grp, long *ulist)
       if (grp == (long)(w->pw_gid))
 	{
 #ifdef TRUST_DEBUG
-	  fprintf(stderr, "USER: %s %d\n", w->pw_name, (int)w->pw_uid); 
+	  fprintf(stderr, "trustfile: checking group member %s, uid %d\n", 
+		  w->pw_name, (int)w->pw_uid); 
 #endif
 	  /* is it a trusted user ?
 	   */
@@ -436,13 +526,30 @@ int onlytrustedingrp(long grp, long *ulist)
 	  for(u = ulist; *u != -1; u++)
 	    {
 	      if (*u == (long)(w->pw_uid))
-		flag = 0;
+		{
+#ifdef TRUST_DEBUG
+		  fprintf (stderr, 
+			   "trustfile: uid=%ld, trusted_uid=%ld, match found --> OK\n", 
+			   (long)(w->pw_uid), *u);
+#endif 
+		  flag = 0;
+		  break;
+		}
+	      else
+		{
+#ifdef TRUST_DEBUG
+		  fprintf (stderr, 
+			   "trustfile: uid=%ld, trusted_uid=%ld, no match\n", 
+			   (long)(w->pw_uid), *u);
+#endif 
+		  ;
+		}
 	    }
 	  /* not found */
 	  if (flag == -1)
 	    {
 #ifdef TRUST_DEBUG
-	      fprintf(stderr, "USER: not found in trusted users\n"); 
+	      fprintf(stderr,"trustfile: group member %s not found in trusted users --> ERROR\n", w->pw_name); 
 #endif
 	      SL_IRETURN(SL_FALSE, _("onlytrustedingrp"));
 	    }
@@ -458,6 +565,11 @@ int onlytrustedingrp(long grp, long *ulist)
   /* #ifdef HAVE_GETPWENT */
 #endif
 
+#ifdef TRUST_DEBUG
+  fprintf(stderr,
+	  "trustfile: group %ld:  all members are trusted users --> OK\n", 
+	  grp);
+#endif
   /* all found
    */
   SL_IRETURN(SL_TRUE, _("onlytrustedingrp"));
@@ -478,8 +590,14 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
    * next expand to the full file name
    * getfname sets sl_errno as appropriate
    */
+#ifdef TRUST_MAIN
+  sl_errno = getfname(fname, fexp, MAXFILENAME);
+  if (sl_errno != 0)
+    return sl_errno;
+#else
   if (SL_ISERROR(getfname(fname, fexp, MAXFILENAME)))
       SL_IRETURN(sl_errno, _("sl_trustfile"));
+#endif
 
   if (okusers == NULL && badusers == NULL)
     {
@@ -522,6 +640,12 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
       if (retry_lstat(FIL__, __LINE__, fexp, &stbuf) < 0)
 	{
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
+#ifdef TRUST_MAIN
+	  fprintf(stderr, "---------------------------------------------\n");
+	      fprintf(stderr, "trustfile: ESTAT: stat(%s) failed, maybe the file does not exist\n",
+		      fexp);
+	  fprintf(stderr, "---------------------------------------------\n");
+#endif
 	  SL_IRETURN(SL_ESTAT, _("sl_trustfile"));
 	}
 
@@ -566,7 +690,15 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 	  if (lsym >= 0) 
 	    csym[lsym] = '\0';
 	  else
-	    SL_IRETURN(SL_EBADNAME, _("sl_trustfile"));
+	    {
+#ifdef TRUST_MAIN
+	      fprintf(stderr, "---------------------------------------------\n");
+	      fprintf(stderr, "trustfile: EBADNAME: readlink(%s) failed\n",
+		      fexp);
+	      fprintf(stderr, "---------------------------------------------\n");
+#endif
+	      SL_IRETURN(SL_EBADNAME, _("sl_trustfile"));
+	    }
 
 	  /*
 	   * relative or absolute referent?
@@ -602,6 +734,13 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 		  /* yes -- error 
 		   */
 		  (void) strcpy(tf_path, fexp);          /* known to fit  */
+#ifdef TRUST_MAIN
+		  fprintf(stderr, "---------------------------------------------\n");
+		  fprintf(stderr, 
+			  "trustfile: ETRUNC: normalized path too long (%s)\n",
+			  fexp);
+		  fprintf(stderr, "---------------------------------------------\n");
+#endif
 		  SL_IRETURN(SL_ETRUNC, _("sl_trustfile"));
 		}
 	      *b = '\0';
@@ -641,6 +780,9 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 #endif
 
 			
+#ifdef TRUST_DEBUG
+      fprintf(stderr, "\ntrustfile: checking path=%s\n", fexp); 
+#endif 
       /*
        * if the owner is not trusted then -- as the owner can
        * change protection modes -- he/she can write to the
@@ -650,7 +792,15 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 	   (badusers != NULL && SL_TRUE == isin((long)stbuf.st_uid,badusers))))
 	{
 #ifdef TRUST_DEBUG
-	  fprintf(stderr, "FIXME: EBADUID %s\n", fexp); 
+	  fprintf(stderr, "---------------------------------------------\n");
+	  fprintf(stderr, "trustfile: EBADUID %s (owner not trusted)\n", 
+		  fexp); 
+	  fprintf(stderr, "The owner of this file/directory is not in samhains\n"); 
+	  fprintf(stderr, "list of trusted users.\n");
+	  fprintf(stderr, "Please run ./configure again with the option\n");
+	  fprintf(stderr, " ./configure [more options] --with-trusted=0,...,UID\n"); 
+	  fprintf(stderr, "where UID is the UID of the (yet) untrusted user.\n"); 
+	  fprintf(stderr, "---------------------------------------------\n");
 #endif 
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
 	  SL_IRETURN(SL_EBADUID, _("sl_trustfile"));
@@ -677,7 +827,16 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 	  )
 	{
 #ifdef TRUST_DEBUG
-	  fprintf(stderr, "FIXME: EBADGID %d %s\n", (int)stbuf.st_gid, fexp);
+	  fprintf(stderr, "---------------------------------------------\n");
+	  fprintf(stderr, 
+		  "trustfile: EBADGID %d %s (group member not trusted)\n", 
+		  (int)stbuf.st_gid, fexp);
+	  fprintf(stderr, "This file/directory is group writeable, and one of the group members\n");
+	  fprintf(stderr, "is not in samhains list of trusted users.\n"); 
+	  fprintf(stderr, "Please run ./configure again with the option\n");
+	  fprintf(stderr, " ./configure [more options] --with-trusted=0,...,UID\n"); 
+	  fprintf(stderr, "where UID is the UID of the (yet) untrusted user.\n"); 
+	  fprintf(stderr, "---------------------------------------------\n");
 #endif 
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
 	  SL_IRETURN(SL_EBADGID, _("sl_trustfile"));
@@ -693,6 +852,13 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 #endif
 	  )
 	{
+#ifdef TRUST_DEBUG
+	  fprintf(stderr, "---------------------------------------------\n");
+	  fprintf(stderr, "trustfile: EBADOTH (world writeable): %s\n", 
+		  fexp);
+	  fprintf(stderr, "This file/directory is world writeable.\n");
+	  fprintf(stderr, "---------------------------------------------\n");
+#endif 
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
 	  SL_IRETURN(SL_EBADOTH, _("sl_trustfile"));
 	}
@@ -720,3 +886,29 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
   SL_IRETURN(SL_ENONE, _("sl_trustfile"));
 }
 
+#ifdef TRUST_MAIN
+int main (int argc, char * argv[])
+{
+  int status;
+
+  if (argc < 2) {
+    fprintf(stderr, "%s: Usage: %s <fullpath>\n", argv[0], argv[0]);
+    return 1;
+  }
+
+  tf_path[0] = '\0';
+  tf_euid = geteuid();
+
+  status = sl_trustfile(argv[1], NULL, NULL);
+  if (status != SL_ENONE)
+    {
+      fprintf(stderr, "trustfile: ERROR: not a trusted path: %s\n",
+	      argv[1]);
+      return 1;
+    }
+  return 0;
+}
+#endif
+
+
+
